@@ -26,8 +26,9 @@
 
 $this->includeJsFile('configuration.templategroup.list.js.php');
 
-$widget = (new CWidget())
+$html_page = (new CHtmlPage())
 	->setTitle(_('Template groups'))
+	->setDocUrl(CDocHelper::getUrl(CDocHelper::DATA_COLLECTION_TEMPLATE_GROUPS_LIST))
 	->setControls((new CTag('nav', true, (new CList())
 		->addItem(CWebUser::getType() == USER_TYPE_SUPER_ADMIN
 			? (new CSimpleButton(_('Create template group')))
@@ -148,7 +149,7 @@ $form->addItem([
 	], 'templategroup')
 ]);
 
-$widget
+$html_page
 	->addItem($filter)
 	->addItem($form)
 	->show();
@@ -156,7 +157,7 @@ $widget
 (new CScriptTag('view.init('.json_encode([
 	'delete_url' => (new CUrl('zabbix.php'))
 		->setArgument('action', 'templategroup.delete')
-		->setArgumentSID()
+		->setArgument(CCsrfTokenHelper::CSRF_TOKEN_NAME, CCsrfTokenHelper::get('templategroup'))
 		->getUrl()
 ]).');'))
 	->setOnDocumentReady()
