@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
  */
 
 $form = (new CForm())
-	->cleanItems()
 	->setName('dashboard_properties_form')
 	->addItem(getMessages());
 
@@ -86,6 +85,7 @@ $form->addItem($form_list);
 
 $output = [
 	'header' => _('Dashboard properties'),
+	'doc_url' => CDocHelper::getUrl(CDocHelper::DASHBOARDS_PROPERTIES_EDIT),
 	'body' => $form->toString(),
 	'buttons' => [
 		[
