@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -32,16 +32,19 @@
 	const ZBX_STYLE_MACRO_VALUE_SECRET = 'macro-value-secret';
 	const ZBX_STYLE_MACRO_VALUE_VAULT = 'macro-value-vault';
 
+	const ZBX_STYLE_ICON_INVISIBLE = 'icon-invisible';
+	const ZBX_STYLE_ICON_SECRET = 'icon-secret';
+
 	function btnUndoFocusEventHandle() {
 		$(this)
-			.closest('.input-group')
+			.closest('.macro-input-group')
 			.find('.btn-undo')
 			.toggleClass('is-focused');
 	}
 
 	function btnUndoClickEventHandle() {
 		var $this = $(this),
-			$container = $this.closest('.input-group'),
+			$container = $this.closest('.macro-input-group'),
 			$input_container = $('.input-secret, .textarea-flexible', $container),
 			$input = $('.input-secret input[type=password], .textarea-flexible', $container),
 			$dropdown_value = $('.dropdown-value', $container);
@@ -74,7 +77,9 @@
 			.val(ZBX_MACRO_TYPE_SECRET)
 			.trigger('change');
 
-		$('.btn-dropdown-container button', $container).addClass('btn-alt btn-dropdown-toggle icon-secret');
+		$('.btn-dropdown-container button', $container)
+			.removeClass('icon-text ' + ZBX_STYLE_ICON_SECRET)
+			.addClass('btn-alt btn-dropdown-toggle ' + ZBX_STYLE_ICON_INVISIBLE);
 
 		$this.hide();
 	}
@@ -94,7 +99,7 @@
 	function inputDropdownValueChangeEventHandle() {
 		var $this = $(this),
 			value_type = $this.val(),
-			$container = $this.closest('.input-group'),
+			$container = $this.closest('.macro-input-group'),
 			curr_value_type = getCurrentValueType($container);
 
 		if (value_type == curr_value_type) {
@@ -152,7 +157,7 @@
 				.attr({
 					id: $input.attr('id'),
 					name: $input.attr('name'),
-					placeholder: (value_type == ZBX_MACRO_TYPE_VAULT) ? t('path/to/secret:key') : t('value'),
+					placeholder: t('value'),
 					maxlength: $input.attr('maxlength')
 				})
 				.text($input.is(':disabled') ? '' : $input.val())
@@ -166,7 +171,7 @@
 	var methods = {
 		init() {
 			return this.each(function () {
-				$('.input-secret input, .input-group .textarea-flexible', $(this))
+				$('.input-secret input, .macro-input-group .textarea-flexible', $(this))
 					.off('focus blur', btnUndoFocusEventHandle)
 					.on('focus blur', btnUndoFocusEventHandle);
 				$('.btn-undo', $(this))
